<?php

namespace Superrobota\Hardskill\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Superrobota\Hardskill\Http\Requests\HardskillRequest;
use Superrobota\Hardskill\Interfaces\HardskillRepositoryInterface;
use Superrobota\Hardskill\Models\Hardskill;

/**
 * Resource controller class for hardskill.
 */
class HardskillResourceController extends BaseController
{

    /**
     * Initialize hardskill resource controller.
     *
     * @param type HardskillRepositoryInterface $hardskill
     *
     * @return null
     */
    public function __construct(HardskillRepositoryInterface $hardskill)
    {
        parent::__construct();
        $this->repository = $hardskill;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Superrobota\Hardskill\Repositories\Criteria\HardskillResourceCriteria::class);
    }

    /**
     * Display a list of hardskill.
     *
     * @return Response
     */
    public function index(HardskillRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Superrobota\Hardskill\Repositories\Presenter\HardskillPresenter::class)
                ->$function();
        }

        $hardskills = $this->repository->paginate();

        return $this->response->title(trans('hardskill::hardskill.names'))
            ->view('hardskill::hardskill.index', true)
            ->data(compact('hardskills'))
            ->output();
    }

    /**
     * Display hardskill.
     *
     * @param Request $request
     * @param Model   $hardskill
     *
     * @return Response
     */
    public function show(HardskillRequest $request, Hardskill $hardskill)
    {

        if ($hardskill->exists) {
            $view = 'hardskill::hardskill.show';
        } else {
            $view = 'hardskill::hardskill.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('hardskill::hardskill.name'))
            ->data(compact('hardskill'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new hardskill.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(HardskillRequest $request)
    {

        $hardskill = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('hardskill::hardskill.name')) 
            ->view('hardskill::hardskill.create', true) 
            ->data(compact('hardskill'))
            ->output();
    }

    /**
     * Create new hardskill.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(HardskillRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $hardskill                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('hardskill::hardskill.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('hardskill/hardskill/' . $hardskill->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/hardskill/hardskill'))
                ->redirect();
        }

    }

    /**
     * Show hardskill for editing.
     *
     * @param Request $request
     * @param Model   $hardskill
     *
     * @return Response
     */
    public function edit(HardskillRequest $request, Hardskill $hardskill)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('hardskill::hardskill.name'))
            ->view('hardskill::hardskill.edit', true)
            ->data(compact('hardskill'))
            ->output();
    }

    /**
     * Update the hardskill.
     *
     * @param Request $request
     * @param Model   $hardskill
     *
     * @return Response
     */
    public function update(HardskillRequest $request, Hardskill $hardskill)
    {
        try {
            $attributes = $request->all();

            $hardskill->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('hardskill::hardskill.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('hardskill/hardskill/' . $hardskill->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('hardskill/hardskill/' . $hardskill->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the hardskill.
     *
     * @param Model   $hardskill
     *
     * @return Response
     */
    public function destroy(HardskillRequest $request, Hardskill $hardskill)
    {
        try {

            $hardskill->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('hardskill::hardskill.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('hardskill/hardskill/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('hardskill/hardskill/' . $hardskill->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple hardskill.
     *
     * @param Model   $hardskill
     *
     * @return Response
     */
    public function delete(HardskillRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('hardskill::hardskill.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('hardskill/hardskill'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/hardskill/hardskill'))
                ->redirect();
        }

    }

    /**
     * Restore deleted hardskills.
     *
     * @param Model   $hardskill
     *
     * @return Response
     */
    public function restore(HardskillRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('hardskill::hardskill.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/hardskill/hardskill'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/hardskill/hardskill/'))
                ->redirect();
        }

    }

}
