<?php

namespace Superrobota\Hardskill\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Superrobota\Hardskill\Http\Requests\PositionRequest;
use Superrobota\Hardskill\Interfaces\PositionRepositoryInterface;
use Superrobota\Hardskill\Models\Position;

/**
 * Resource controller class for position.
 */
class PositionResourceController extends BaseController
{

    /**
     * Initialize position resource controller.
     *
     * @param type PositionRepositoryInterface $position
     *
     * @return null
     */
    public function __construct(PositionRepositoryInterface $position)
    {
        parent::__construct();
        $this->repository = $position;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Superrobota\Hardskill\Repositories\Criteria\PositionResourceCriteria::class);
    }

    /**
     * Display a list of position.
     *
     * @return Response
     */
    public function index(PositionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Superrobota\Hardskill\Repositories\Presenter\PositionPresenter::class)
                ->$function();
        }

        $positions = $this->repository->paginate();

        return $this->response->title(trans('hardskill::position.names'))
            ->view('hardskill::position.index', true)
            ->data(compact('positions'))
            ->output();
    }

    /**
     * Display position.
     *
     * @param Request $request
     * @param Model   $position
     *
     * @return Response
     */
    public function show(PositionRequest $request, Position $position)
    {

        if ($position->exists) {
            $view = 'hardskill::position.show';
        } else {
            $view = 'hardskill::position.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('hardskill::position.name'))
            ->data(compact('position'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new position.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PositionRequest $request)
    {

        $position = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('hardskill::position.name')) 
            ->view('hardskill::position.create', true) 
            ->data(compact('position'))
            ->output();
    }

    /**
     * Create new position.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PositionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $position                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('hardskill::position.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('hardskill/position/' . $position->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/hardskill/position'))
                ->redirect();
        }

    }

    /**
     * Show position for editing.
     *
     * @param Request $request
     * @param Model   $position
     *
     * @return Response
     */
    public function edit(PositionRequest $request, Position $position)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('hardskill::position.name'))
            ->view('hardskill::position.edit', true)
            ->data(compact('position'))
            ->output();
    }

    /**
     * Update the position.
     *
     * @param Request $request
     * @param Model   $position
     *
     * @return Response
     */
    public function update(PositionRequest $request, Position $position)
    {
        try {
            $attributes = $request->all();

            $position->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('hardskill::position.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('hardskill/position/' . $position->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('hardskill/position/' . $position->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the position.
     *
     * @param Model   $position
     *
     * @return Response
     */
    public function destroy(PositionRequest $request, Position $position)
    {
        try {

            $position->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('hardskill::position.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('hardskill/position/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('hardskill/position/' . $position->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple position.
     *
     * @param Model   $position
     *
     * @return Response
     */
    public function delete(PositionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('hardskill::position.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('hardskill/position'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/hardskill/position'))
                ->redirect();
        }

    }

    /**
     * Restore deleted positions.
     *
     * @param Model   $position
     *
     * @return Response
     */
    public function restore(PositionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('hardskill::position.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/hardskill/position'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/hardskill/position/'))
                ->redirect();
        }

    }

}
